###ML bias package
###############machine learning bias method using lots of real datasets


# data=iris
# dataX=iris[,1:4]
# dataY=ifelse(iris[,5]=="setosa",1,0)
# dataY=sample(dataY)
# dataX=matrix(rnorm(200),nrow=50,4)
# dataY=sample(c(rep(1,25),rep(0,25)))
# ####
###the current chosen methods are 
##1-3 knn k=1,3,5
##4 diagonal linear discriminant analysis
##5 random forest
##6 neural network
##7 support vector machine with linear kernel
##8 support vector machine with radial kernel (nonlinear)
##9 fisher's discriminant analysis
##10 PAM



ML_bias<-function(dataX, dataY, cross_validation="5_fold", feature_set=c(2:3),
                  cpu_num=4){
  #library(impute)
  options(stringsAsFactors = FALSE);
  set.seed(1234)
  #setwd("/home/dingying85/Desktop/machine_learning_bias/data/")
  library(CMA)
  
  library(DEoptim)
  classifiers=c("knn k=1","knn k=3","knn k=5","diagonal linear discriminant analysis",
                "random forest","neural network","support vector machine with linear kernel",
                "support vector machine with radial kernel","fisher's discriminant analysis",
                "PAM")
  
#machine learning bias fitting
#this code is for obtaining the data for fitting for different classifier
##############################
####  function
if(cross_validation%in%c("5_fold","10_fold")==FALSE){
  stop("cross validation for this method can only be 5_fold or 10_fold!")
}
if(max(feature_set)>dim(dataX)[2]){
  stop("feature number can not exceed the total number of features in the dataset!")
}


if(cross_validation=="5_fold"){
  fold_num=5
}else if(cross_validation=="10_fold"){
  fold_num=10
}


library(CMA)
library(rda)
library(e1071)
library(limma)
library(randomForest)
library(Biobase)
library(pls)


#############
###tibshirani estimate
tibshi<-function(dataX,dataY,seed,fold_num,feature_set){
  set.seed(seed)
  library(CMA)
  library(rda)
  library(e1071)
  library(limma)
  library(randomForest)
  library(Biobase)
  library(pls)
  library(mvtnorm) ##multivariable normal generator
  library(plsgenomics)
  

    dataX=as.matrix(dataX)
    dataY=as.factor(dataY)
    lset=GenerateLearningsets(y=dataY,method="CV",fold=fold_num,strat=TRUE)
    selttest=GeneSelection(X=as.matrix(dataX), y=dataY, learningsets = lset, method="t.test")
   
  num_of_classifiers=10*length(feature_set);
  error_rate=matrix(0,nrow=num_of_classifiers,ncol=fold_num)
  index_select=1:num_of_classifiers
  cf_list=rep(1:10,each=length(feature_set))
  pair_list=paste(cf_list,feature_set)
  for(x in 1:num_of_classifiers){
    print(x)
    feature_index=x%%length(feature_set)
    if(feature_index==0){
      feature_index=length(feature_set)
    }
    feature_num=feature_set[feature_index]
    print(feature_num)
    data_X=dataX
    data_Y=dataY
    sqrt_num<-round(sqrt(length(data_Y)))
    index=cf_list[x]       
    if(index==1){      
      ###### knn k=1 ######      
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 genesel = selttest, classifier = knnCMA,k = 1, nbgene = feature_num))@score
    }
    ### knn k=3 ###
    if(index==2){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 genesel = selttest, classifier = knnCMA,
                                                 k = 3, nbgene = feature_num))@score
    }
                   
    ### knn k=5 ###
    if(index==3){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 genesel = selttest, classifier = knnCMA,
                                                 k = 5, nbgene = feature_num))@score
    }
    ### dlda ###
    
    if(index==4){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 genesel=selttest, classifier = dldaCMA, nbgene = feature_num))@score
    }
    ### nnet ###
    if(index==5){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 genesel=selttest, classifier = nnetCMA, nbgene=feature_num))@score
    }
   
    if(index==6){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 classifier = rfCMA,genesel=selttest,nbgene=feature_num))@score
    }
    if(index==7){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 classifier = svmCMA,kernel="linear",genesel=selttest,nbgene=feature_num))@score  
    }
    if(index==8){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 classifier = svmCMA,kernel="radial",genesel=selttest,nbgene=feature_num))@score  
    }
    if(index==9){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 classifier = fdaCMA,genesel=selttest,nbgene=feature_num))@score  
    }
    if(index==10){
      error_rate[x,] = evaluation(classification(data_X, data_Y, learningsets = lset,
                                                 classifier = scdaCMA,genesel=selttest,nbgene=feature_num))@score  
    }
   
  }
  
  
  tib_estimate=min(apply(error_rate,1,mean))*2-mean(apply(error_rate,2,min))
  MEC_error=min(apply(error_rate,1,mean))
  index_best=which.min(apply(error_rate,1,mean))
  result=list(MEC_error=MEC_error,tib=tib_estimate,error_rate=rowMeans(error_rate),best_classifier=pair_list[index_best])
  
  return(result)
  
}  


tibshi_estimate=tibshi(dataX,dataY,123,fold_num,feature_set)
index1=as.numeric(strsplit(tibshi_estimate$best_classifier,split=" ")[[1]][1])
index2=as.numeric(strsplit(tibshi_estimate$best_classifier,split=" ")[[1]][2])


# ######################
# ####nestedCV estimtate
# ######################
nested_CV<-function(dataX,dataY,seed,feature_set){
  set.seed(seed)
  library(CMA)
  library(rda)
  library(e1071)
  library(limma)
  library(randomForest)
  library(Biobase)
  library(pls)
  library(mvtnorm) ##multivariable normal generator
  library(plsgenomics)
  
  #num_of_classifiers=35
  dataX=as.matrix(dataX)
  dataY=as.factor(dataY)
    if(length(dataY)<=40){
      lset=GenerateLearningsets(y=dataY,method="LOOCV")
      fold_num=length(dataY)      
    }else{
      fold_num=10
      lset=GenerateLearningsets(y=dataY,method="CV",fold=fold_num,strat=TRUE)
    }
    learnmatrix1=attributes(lset)$learnmatrix
    error_rate_summary=array(0,dim=fold_num)
    for(i in 1:fold_num){
      trainX=dataX[learnmatrix1[i,],]
      trainY=dataY[learnmatrix1[i,]]
      testX=dataX[-learnmatrix1[i,],]
      testY=dataY[-learnmatrix1[i,]]
     
      if(length(trainY)<=80){
        lset=GenerateLearningsets(y=factor(trainY),method = "LOOCV")
      }else{
        lset=GenerateLearningsets(y=factor(trainY),method = "CV", fold=10)
      }      
      sqrt_num<-round(sqrt(length(trainY)))
      selttest=GeneSelection(X=trainX, y=trainY, learningsets = lset, method="t.test")
      #select=c(1:10);
      num_of_classifiers=10*length(feature_set);
      error_rate=array(0,dim=num_of_classifiers);
      index_select=1:num_of_classifiers
      cf_list=array(sapply(1:10,function(x) rep(x,times=length(feature_set))))
      for(x in 1:num_of_classifiers){ 
        #index=x
        print(x)
        feature_index=x%%length(feature_set)
        if(feature_index==0){
          feature_index=length(feature_set)
        }
        feature_num=feature_set[feature_index]
        index=cf_list[x]
        if(index==1){      
          ###### knn k=1 ######      
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     genesel = selttest, classifier = knnCMA,k = 1, nbgene = feature_num))@score)
        }      
        ### knn k=3 ###
        if(index==2){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     genesel = selttest, classifier = knnCMA,
                                                     k = 3, nbgene = feature_num))@score)
        }       
        ### knn k=5 ###
        if(index==3){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     genesel = selttest, classifier = knnCMA,
                                                     k = 5, nbgene = feature_num))@score)
        }  
        ### dlda ###       
        if(index==4){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     genesel=selttest, classifier = dldaCMA, nbgene = feature_num))@score)
        }
        ### nnet ###
        if(index==5){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     genesel=selttest, classifier = nnetCMA, nbgene=feature_num))@score)
        }
       
        if(index==6){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     classifier = rfCMA,genesel=selttest,nbgene=feature_num))@score)
        }    
        if(index==7){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                     classifier = svmCMA,kernel="linear",genesel=selttest,nbgene=feature_num))@score)  
        }
        if(index==8){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                         genesel=selttest, classifier = svmCMA,kernel="radial",nbgene=feature_num))@score)
        }
        
        if(index==9){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                         classifier = fdaCMA,genesel=selttest,nbgene=feature_num))@score)
        }    
        if(index==10){
          error_rate[x] = mean(evaluation(classification(trainX, trainY, learningsets = lset,
                                                         classifier = scdaCMA,genesel=selttest,nbgene=feature_num))@score)  
        }
        
      }
      index_best=which.min(error_rate)
      #index=select[index];   
      ##construct learning sets instead of using GenerateLearningSets function
      learnmatrix<-matrix(1:length(trainY),nrow=1)
      X=rbind(trainX,testX)
      Y=c(trainY,testY)
      lset<-new("learningsets",learnmatrix=learnmatrix, method= "my method", ntrain=ncol(learnmatrix),
                iter=nrow(learnmatrix))
      
      sqrt_num<-round(sqrt(length(Y)))
      selttest=GeneSelection(X=X, y=Y, learningsets = lset, method="t.test")      
      feature_index=index_best%%length(feature_set)
      if(feature_index==0){
        feature_index=length(feature_set)
      }
      feature_num=feature_set[feature_index]
      index_best1=cf_list[index_best]     
      if(index_best1==1){      
        ###### knn k=1 ######      
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,k = 1, nbgene = feature_num))@score)
      }
      ### knn k=3 ###
      if(index_best1==2){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,
                                                       k = 3, nbgene = feature_num))@score)
      }      
      ### knn k=5 ###
      if(index_best1==3){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,
                                                       k = 5, nbgene = feature_num))@score)
      }      
      ### dlda ###      
      if(index_best1==4){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel=selttest, classifier = dldaCMA, nbgene = feature_num))@score)
      }
      ### nnet ###
      if(index_best1==5){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel=selttest, classifier = nnetCMA, nbgene=feature_num))@score)
      }     
      if(index_best1==6){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       classifier = rfCMA,genesel=selttest,nbgene=feature_num))@score)
      }      
      if(index_best1==7){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       classifier = svmCMA,kernel="linear",genesel=selttest,nbgene=feature_num))@score)  
      }
      ### nnet ###
      if(index_best1==8){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                               genesel=selttest, classifier = svmCMA,kernel="radial", 
                                                               nbgene=feature_num))@score)
      }     
      if(index_best1==9){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                               classifier = fdaCMA,genesel=selttest,nbgene=feature_num))@score)
      }      
      if(index_best1==10){
        error_rate_summary[i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                               classifier = scdaCMA,genesel=selttest,nbgene=feature_num))@score)  
      }
      
      
    }
     
  error_rate_full=mean(error_rate_summary)
  return(error_rate_full)
  
}  

nestedCV_estimate=nested_CV(dataX,dataY,123,feature_set)


########################################################
#####IPL method
#######################################################
IPL_estimate<-function(dataX,dataY,seed,subsampling_repeat,sub_sample_size,index,feature_set){
  library(CMA)
  library(rda)
  library(e1071)
  library(limma)
  library(randomForest)
  library(Biobase)
  library(pls)
  library(mvtnorm) ##multivariable normal generator
  library(plsgenomics)    
  #########################
  ###specify the parameters
  ########################
  set.seed(seed)##set seed for reuse
  significance<-0.15 #significance level to test classifiers
  result<-vector("list",length=1)                             
  ##B subsampling####
  #subsampling_repeat=2;
  dataX=as.matrix(dataX)
  dataY=as.factor(dataY)
  #sample_size_list=c(20,21,22,24,26,28,30,32,33)
  points_num<-length(sample_size_list)
  data_points<-matrix(0,points_num,2) ##matrix for fitting the lines, select 9 points
  method_num=10;
  num_of_classifiers=10*length(feature_set);
  error_rate<-matrix(0,nrow=num_of_classifiers,ncol=subsampling_repeat)
  #error_rate_permute<-matrix(0,nrow=subsampling_repeat,ncol=permutation_time)
  #num_of_samples<-150 #total number of samples
  round=0  
    class1_size=round(sub_sample_size*sum(dataY==0)/length(dataY))
    class2_size=sub_sample_size-class1_size  
#     round<-round+1    ###round indicate the different subsample size
    #############    
    #error_rate<-array(0,dim=subsampling_repeat)
    for(i in 1:subsampling_repeat){
      print(i)
      ########################
      ##when dataset is fixed
      ########################
      label1=sample(which(dataY==0),class1_size,replace=FALSE)
#       if(ttt==max(sample_size_list)){
#         label1<-sample(1:fixed_training_class1_size,resample_fixed_training_class1_size,replace=TRUE)
#       }
      label2=sample(which(dataY==1),class2_size,replace=FALSE)      
      X<-dataX[c(label1,label2),]
      Y<-dataY[c(label1,label2)]      
#       if(length(Y)<30){
#         lset=GenerateLearningsets(y=Y,method="LOOCV")
#         fold_num=length(Y)      
#       }else{
        fold_num=5
        lset=GenerateLearningsets(y=Y,method="CV",fold=fold_num,strat=TRUE)
#      }
     # learnmatrix1=attributes(lset)$learnmatrix      
      sqrt_num<-round(sqrt(length(Y)))
      selttest=GeneSelection(X=X, y=Y, learningsets = lset, method="t.test")
      
      
      #index_select=1:num_of_classifiers
      cf_list=array(sapply(1:10,function(x) rep(x,times=length(feature_set))))
      for(index in 1:num_of_classifiers){
      method_index=cf_list[index]
      feature_index=index%%length(feature_set)
      if(feature_index==0){
        feature_index=length(feature_set)
      }
      feature_num=feature_set[feature_index]
      if(method_index==1){      
        ###### knn k=1 ######      
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,k = 1, nbgene = feature_num))@score)
      }
      ### knn k=3 ###
      if(method_index==2){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,
                                                       k = 3, nbgene = feature_num))@score)
      }
      ### knn k=5 ###
      if(method_index==3){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel = selttest, classifier = knnCMA,
                                                       k = 5, nbgene = feature_num))@score)
      }    
      ### dlda ###      
      if(method_index==4){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel=selttest, classifier = dldaCMA, nbgene = feature_num))@score)
      }
      ### nnet ###
      if(method_index==5){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       genesel=selttest, classifier = nnetCMA, nbgene=feature_num))@score)
      }
      if(method_index==6){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       classifier = rfCMA,genesel=selttest,nbgene=feature_num))@score)
      }
      if(method_index==7){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                       classifier = svmCMA,kernel="linear",genesel=selttest,nbgene=feature_num))@score)  
      }         
      if(method_index==8){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                           genesel=selttest, classifier = svmCMA,kernel="radial", 
                                                           nbgene=feature_num))@score)
      }
      if(method_index==9){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                           classifier = fdaCMA,genesel=selttest,nbgene=feature_num))@score)
      }
      if(method_index==10){
        error_rate[index,i] = mean(evaluation(classification(X, Y, learningsets = lset,
                                                           classifier = scdaCMA,genesel=selttest,nbgene=feature_num))@score)  
      }         
    }
      
      
    }
    error_rate_mean=rowMeans(error_rate)

  
  return(error_rate_mean)
}   

sample_size_list=c(30,31,32,seq(from=33,to=length(dataY)-1,by=3))
subsampling_repeat=30
seed=123
#num_of_classifiers=35;
num_of_classifiers=10*length(feature_set)
#snow_fun(1,X,Y)
library(snowfall)
ptm<-proc.time()
sfInit(parallel=TRUE,type="SOCK", cpus=cpu_num)
seed=123
sfExport("dataX","dataY","seed","subsampling_repeat","sample_size_list","num_of_classifiers","feature_set","IPL_estimate")
result=sfSapply(1:length(sample_size_list),function(x) try(IPL_estimate(dataX,dataY,seed=seed,subsampling_repeat=subsampling_repeat,
                                                              sub_sample_size=sample_size_list[x],index=x,feature_set=feature_set)))
sfStop();



estimate_mean=NULL;
estimate_upper=NULL;
estimate_lower=NULL;
data_point_num=length(dataY)
data_points_compare=c(length(dataY),length(dataY)+20,length(dataY)+40)
#true_min_estimate=matrix(0,nrow=index_num,ncol=data_points_compare_num)
num_of_sample=length(dataY)      #####total number of samples

summary_result=result

  total_num<-dim(summary_result)[1]
  #result<-summary_result[[k]]
  best_value<-matrix(0,total_num,3)    ####fitting parameters for mean
  best_value_lower<-matrix(0,total_num,3)########fitting parameters for lower quantile
  best_value_upper<-matrix(0,total_num,3)#########fitting parameters for upper quantile
  lkh<-function(x){
    nl<-data_points_final[,1]
    error<-data_points_final[,2]
    return(sum((x[1]*nl^(-x[2])+x[3]-error)^2))
  }
  lkh1<-function(x,data_points_list){
    nl<-data_points_list  ###use all points in data_points_list
    return(x[1]*nl^(-x[2])+x[3])
  }  
  data_points_list=sample_size_list
  data_points=data_points_list
  ####data points length among the total_num####
  
  #p_value_cutoff=0.15
  data_points_length<-array(0,dim=total_num)
  #par(mfrow=c(3,4))
  for(i in 1:total_num){
    #p_value=summary_result[[index[i]]]$p_value
    cut_index1=1   
    index_p_value=1:length(data_points)  
    data_points_length[i]<-length(index_p_value)    
    data_points_final<-cbind(sample_size_list,summary_result[i,])
    outOptim<-DEoptim(lkh,lower=c(0,0,0),upper=c(5,5,5))###stochastic fitting
    best_value[i,]<-outOptim$optim$bestmem
#          plot(data_points_final[,1],data_points_final[,2],xlab="subsample size",ylab="error rate"
#           ,ylim=c(0,max(data_points_final[,2])+0.1),
#              main=classifiers_names[i],
#              xlim=c(min(data_points_list),max(data_points_list)))
#         lines(data_points_list,lkh1(best_value[i,],data_points_list))
#     data_points_final<-result[[index[i]]]$data_points_lower
#     outOptim<-DEoptim(lkh,lower=c(0,0,0),upper=c(10,10,10)) ###stochastic fitting
#     best_value_lower[i,]<-outOptim$optim$bestmem
#     lines(data_points_list,lkh1(best_value_lower[i,],data_points_list),lty=3)
#     data_points_final<-result[[index[i]]]$data_points_upper
#     outOptim<-DEoptim(lkh,lower=c(0,0,0),upper=c(10,10,10)) ###stochastic fitting
#     best_value_upper[i,]<-outOptim$optim$bestmem
#     lines(data_points_list,lkh1(best_value_upper[i,],data_points_list),lty=3)
#    ### fitting for the first quantile
#         }else{
#           data_points_length[i]=0
#         }
    
  }
  good_fitting_index=index_p_value
  true_error_rate_estimate<-matrix(0,length(data_points_compare),total_num)  ###minimum error rate estimated from fitting
  for(i in 1:total_num){
    true_error_rate_estimate[,i]<-t(lkh1(best_value[i,],data_points_compare))   
  }
  rownames(true_error_rate_estimate)=data_points_compare
  index_best=apply(true_error_rate_estimate,1,which.min)
 # print(classifiers_names[index_best])
  IPL_estimate=apply(true_error_rate_estimate,1,min)


cat(paste("classifier:",classifiers[index1],"with",index2,"of features has the smallest MEC error rate:",
          tibshi_estimate$MEC_error,"\nthe corrected error rate with tibshirani's method is :",format(tibshi_estimate$tib,digits=3),
          "\nthe corrected error rate with nestedCV method is :",format(nestedCV_estimate,digits=3),
          "\nthe corrected error rate with IPL method is :",format(IPL_estimate[1],digits=3)))
# print(index1)
# print(index2)

}



#   mmm=match(data_points_compare,data_original)
#   confidence_method=matrix(0,nrow=length(mmm),ncol=dim(result[[1]]$error_rates_original)[2])
#   for(i in 1:dim(result[[1]]$error_rates_original)[2]){
#     temp=sapply(result,function(x) x$error_rates_original[mmm,i])
#     temp1=apply(temp,1,which.min)
#     confidence_method[,i]=temp1
#   }  
#   
#   confidence_estimate=matrix(0,nrow=length(mmm),ncol=methods_num)
#   for(i in 1:length(mmm)){
#     confidence_estimate[i,as.numeric(names(table(confidence_method[i,])))]=table(confidence_method[i,])/dim(result[[1]]$error_rates_original)[2]
#   }
#   IPL_estimate_list[[k]]=true_error_rate_estimate*confidence_estimate
#   IPL_estimate_list2[,k]= rowSums(true_error_rate_estimate*confidence_estimate)
#   true_min_error_rate_estimate<-apply(true_error_rate_estimate,1,min)  ###true minimal error rate estimate from inverse-power law model
#   classifier_chosen[k,]=apply(true_error_rate_estimate,1,which.min)  
#   true_min_estimate[k,]=true_min_error_rate_estimate
# 
#   
# }

#estimate_result=list(tibshi=tibshi_estimate_5_fold,nestedCV=nestedCV_estimate,IPL=IPL_estimate
#)
#save(IPL_estimate,file=paste("IPL_estimate_metabric_final",ttt,".Rdata",sep=""))
  
#save(IPL_estimate,file=paste("IPL_estimate_",filename,".Rdata",sep=""))
    
#}

# 
# save(estimate_result,file="ML_bias_09_22_13.Rdata")

# 
# 
# quit(save="no")
# 
# 
# par(mfrow=c(1,3),mar=c(7,4,2,2))
# 
# 
# par(mfrow=c(1,3),oma=c(7,1,1,1),mar=c(4,4.3,5,2))
# ####classify ER stage
# IPL_error=array(0,dim=10)
# for(i in 1:10){
#   load(paste("IPL_estimate_tcga_er_",i,".Rdata",sep=""))
#   IPL_error[i]=IPL_estimate[1]
# }
# 
# 
# MEC_error=array(0,dim=10)
# tib_error=array(0,dim=10)
# nestedCV_error=array(0,dim=10)
# all_error=list(10)
# for(i in 1:10){
#   load(paste("tib_nestedCV_tcga_er",i,sep=""))
#   MEC_error[i]=tib_nestedCV[[1]]$MEC_error
#   tib_error[i]=tib_nestedCV[[1]]$tib
#   nestedCV_error[[i]]=tib_nestedCV[[2]]
#   all_error[[i]]=tib_nestedCV[[1]]$error_rate
# }
# 
# true_error_all=Reduce("+",all_error)/10
# true_error=min(true_error_all)  
# 
# load("wmc_tcga_er_status.Rdata")
# 
# boxplot(cbind(MEC_error,tib_error,nestedCV_error,IPL_error,wmc_all,wmcs_all),at=1:6+0.5,ylim=c(0,0.3),xlim=c(1,8),
#         las=2,lwd=3, cex.axis=1.2,cex.lab=1.2,
#         ylab="error rate estimate",main="TCGA(ER status)",xaxt="n")
# points(7.5,lwd=3,true_error,col="black",pch=4,cex=2)
# axis(1,at=1:7+0.5,label=c("MEC","BC","nestedCV","IPL","WMC","WMCS","truth"),las=2,cex.lab=1.2,cex.axis=1.5)
# 
# 
# ####classify tcga tumor stage
# IPL_error=array(0,dim=10)
# for(i in 1:10){
# load(paste("IPL_estimate_tcga_tumor_grade_",i,".Rdata",sep=""))
# IPL_error[i]=IPL_estimate[1]
# }
# 
# 
# MEC_error=array(0,dim=10)
# tib_error=array(0,dim=10)
# nestedCV_error=array(0,dim=10)
# all_error=list(10)
# for(i in 1:10){
# load(paste("tib_nestedCV_tcga_tumor_stage",i,sep=""))
# MEC_error[i]=tib_nestedCV[[1]]$MEC_error
# tib_error[i]=tib_nestedCV[[1]]$tib
# nestedCV_error[[i]]=tib_nestedCV[[2]]
# all_error[[i]]=tib_nestedCV[[1]]$error_rate
# }
# 
# true_error_all=Reduce("+",all_error)/10
# true_error=min(true_error_all)  
# 
# load("wmc_tcga_tumor_grade.Rdata")
# 
# boxplot(cbind(MEC_error,tib_error,nestedCV_error,IPL_error,wmc_all,wmcs_all),at=1:6+0.5,ylim=c(0.15,0.55),xlim=c(1,8),
#         las=2,lwd=3, cex.axis=1.2,cex.lab=1.2,
#         ylab="error rate estimate",main="TCGA(tumor stage)",xaxt="n")
# points(7.5,lwd=3,true_error,col="black",pch=4,cex=2)
# axis(1,at=1:7+0.5,label=c("MEC","BC","nestedCV","IPL","WMC","WMCS","truth"),las=2,cex.lab=1.2,cex.axis=1.5)
# 
# 
# 
# 
# ####classify tumor stage
# num_of_part=47
# 
# IPL_error=array(0,dim=num_of_part)
# count=0
# for(i in 1:num_of_part){
#   count=count+1;
#   load(paste("IPL_estimate_metabric_tumor_grade_",i,".Rdata",sep=""))
#   IPL_error[count]=IPL_estimate[1]
# }
# 
# 
# MEC_error=array(0,dim=num_of_part)
# tib_error=array(0,dim=num_of_part)
# nestedCV_error=array(0,dim=num_of_part)
# all_error=list(num_of_part)
# count=0
# for(i in c(1:num_of_part)){
#   count=count+1
#   load(paste("tib_nestedCV_metabric_tumor_stage",i,sep=""))
#   MEC_error[count]=tib_nestedCV[[1]]$MEC_error
#   tib_error[count]=tib_nestedCV[[1]]$tib
#   nestedCV_error[[count]]=tib_nestedCV[[2]]
#   all_error[[count]]=tib_nestedCV[[1]]$error_rate
# }
# 
# true_error_all=Reduce("+",all_error)/num_of_part
# true_error=min(true_error_all)  
# 
# 
# 
# 
# load("wmc_metabric_tumor_grade.Rdata")
# 
# boxplot(cbind(MEC_error,tib_error,nestedCV_error,IPL_error,wmc_all,wmcs_all),at=1:6+0.5,ylim=c(0.05,0.55),xlim=c(1,8),
#         las=2,lwd=3, cex.axis=1.2,cex.lab=1.2,
#         ylab="error rate estimate",main="METABRIC(tumor stage)",xaxt="n")
# points(7.5,lwd=3,true_error,col="black",pch=4,cex=2)
# axis(1,at=1:7+0.5,label=c("MEC","BC","nestedCV","IPL","WMC","WMCS","truth"),las=2,cex.lab=1.2,cex.axis=1.5)


# library(CMA)
# library(rda)
# library(e1071)
# library(limma)
# library(randomForest)
# library(Biobase)
# library(pls)
# library(mvtnorm) ##multivariable normal generator
# library(plsgenomics)
# ##############################
# ###specify the methods used
# ##############################
# 
# 
# #########################
# ###specify the parameters
# ########################
# ########################
# set.seed(27611)##set seed for reuse
# 
# library(snowfall)                            
# 
# 
# sfInit()
# library(snowfall)
# ptm<-proc.time()
# 
# simulation=1000;
# sfInit(parallel=TRUE,type="SOCK", cpus=15)
# result_tib<-sfSapply(1:simulation,tibshi)
# sfStop()













# library(snowfall)
# ptm=proc.time()
# 
# #result=list(length(Y_list))
# sfInit(parallel=TRUE,type="SOCK", cpus=2)
# sfExport("classifier_fitting")
# for(i in 1:length(Y_list)){
#   print(i)
#   X_temp=X_list[[i]]
#   Y_temp=Y_list[[i]]
#   if(length(X_temp)==1){
#     next;
#   }
#   sfExport("X_temp")
#   sfExport("Y_temp")
#   result=try(sfSapply(c(35:38),function(x) classifier_fitting(x,X_temp,Y_temp)))
#   save(result,file=paste("error_rate_",i,".Rdata",sep=""))
# }
# sfStop()

# 
# print(proc.time()-ptm) #the computational cost
# 
# save(result,file="machine_learning_bias_real_data.Rdata")

# 
# files=list.files(".",pattern="tib_nestedCV_GDS")
# files=sort(files)[2:6]
# final_result=NULL
# for(i in 1:length(files)){
# load(files[i])
# MEC_error_rate=tib_nestedCV$tib$MEC_error
# tibshi_5_fold=tib_nestedCV$tib$tib
# tibshi_10_fold=tib_nestedCV$tib10$tib
# nestedCV=tib_nestedCV$nestedCV
# final_result=rbind(final_result,c(MEC_error_rate,tibshi_5_fold,tibshi_10_fold,nestedCV))
# }
# rownames(final_result)=files
# colnames(final_result)=c("MEC_error_rate","tibshi 5 fold","tibshi 10 fold","nestedCV")
# 
# 
# 
# 
# 
# files1=list.files(".",pattern="IPL_estimate_GDS")
# files1=sort(files1)[1:5]
# IPL=NULL
# for(i in 1:length(files1)){
#   load(files1[i])
#   IPL=c(IPL,IPL_estimate[1])
# }
# 
# final_result1=cbind(final_result,IPL)
# write.csv(final_result1,"five_output_dataset_new.csv")
# 
# 
# anno=NULL
# for(i in 1:length(files1)){
#   load(files1[i])
#   #print(dim(DATA$data))
#   anno=rbind(anno,c(dim(DATA$data),DATA$pheno[1,1]))
# }
# 
# colnames(anno)=c("gene number","sample size","GEO number")
# 
# final_result1=cbind(final_result,anno)
# 
# 
# write.csv(final_result1,file="GEO_ML_estimate_gather.csv")
